@@ -1,5 +1,4 @@
 #!/usr/bin/env python
-# coding: utf-8
 
 """
 This module tests the context manager of Bus and Notifier classes
@@ -10,13 +9,16 @@ import can
 
 
 class ContextManagerTest(unittest.TestCase):
-
     def setUp(self):
         data = [0, 1, 2, 3, 4, 5, 6, 7]
-        self.msg_send = can.Message(extended_id=False, arbitration_id=0x100, data=data)
+        self.msg_send = can.Message(
+            is_extended_id=False, arbitration_id=0x100, data=data
+        )
 
     def test_open_buses(self):
-        with can.interface.Bus(bustype='virtual') as bus_send, can.interface.Bus(bustype='virtual') as bus_recv:
+        with can.Bus(interface="virtual") as bus_send, can.Bus(
+            interface="virtual"
+        ) as bus_recv:
             bus_send.send(self.msg_send)
             msg_recv = bus_recv.recv()
 
@@ -24,7 +26,9 @@ class ContextManagerTest(unittest.TestCase):
             self.assertTrue(msg_recv)
 
     def test_use_closed_bus(self):
-        with can.interface.Bus(bustype='virtual') as bus_send, can.interface.Bus(bustype='virtual') as bus_recv:
+        with can.Bus(interface="virtual") as bus_send, can.Bus(
+            interface="virtual"
+        ) as bus_recv:
             bus_send.send(self.msg_send)
 
         # Receiving a frame after bus has been closed should raise a CanException
@@ -32,5 +36,5 @@ class ContextManagerTest(unittest.TestCase):
         self.assertRaises(can.CanError, bus_send.send, self.msg_send)
 
 
-if __name__ == '__main__':
+if __name__ == "__main__":
     unittest.main()
